==> klayout-0.10.tar.gz: klayout-0.10/src/layPlugin.cc <==

/*

  KLayout Layout Viewer
  Copyright (C) 2006 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include "layPlugin.h"

#include "tlClassRegistry.h"
#include "tlException.h"

//  The instance of the PluginDeclaration class registrar
template <> tl::Registrar<lay::PluginDeclaration> *tl::Registrar<lay::PluginDeclaration>::instance = 0;

namespace lay
{

// ----------------------------------------------------------------
//  Plugin implementation

Plugin::Plugin (Plugin *parent)
  : mp_parent (parent)
{
  if (! parent) {
    //  load the root with the default configuration
    for (tl::Registrar<lay::PluginDeclaration>::iterator cls = tl::Registrar<lay::PluginDeclaration>::begin (); cls != tl::Registrar<lay::PluginDeclaration>::end (); ++cls) {
      std::vector< std::pair<std::string, std::string> > pairs;
      cls->get_options (pairs);
      m_repository.insert (pairs.begin (), pairs.end ());
    }
  } else {
    mp_parent->m_children.insert (this, false /*not owned*/);
  }
}

Plugin::~Plugin ()
{
  // .. nothing yet ..
}

void 
Plugin::config_setup ()
{
  do_config_setup (this);
}
  
void 
Plugin::config_set (const std::string &name, const std::string &value)
{
  tl_assert (mp_parent == 0);

  //  on the root, install the value in the repository
  m_repository.insert (std::make_pair (name, std::string ())).first->second = value;

  do_config_set (name, value);
}

void 
Plugin::config_end ()
{
  tl_assert (mp_parent == 0);
  do_config_end ();
}

bool 
Plugin::config_get (const std::string &name, std::string &value) const
{
  tl_assert (mp_parent == 0);

  std::map<std::string, std::string>::const_iterator p = m_repository.find (name);
  if (p != m_repository.end ()) {
    value = p->second;
    return true;
  } else {
    value = "";
    return false;
  }
}

void 
Plugin::get_config_names (std::vector<std::string> &names)
{
  //  retrieve the configuration names
  for (tl::Registrar<lay::PluginDeclaration>::iterator cls = tl::Registrar<lay::PluginDeclaration>::begin (); cls != tl::Registrar<lay::PluginDeclaration>::end (); ++cls) {

    std::vector< std::pair<std::string, std::string> > pairs;
    cls->get_options (pairs);
    for (std::vector< std::pair<std::string, std::string> >::const_iterator p = pairs.begin (); p != pairs.end (); ++p) {
      names.push_back (p->first);
    }

  }
}

void 
Plugin::do_config_setup (Plugin *target)
{
  if (mp_parent) {
    mp_parent->do_config_setup (target);
  } else {
    for (std::map<std::string, std::string>::const_iterator p = m_repository.begin (); p != m_repository.end (); ++p) {
      target->do_config_set (p->first, p->second);
    }
    target->do_config_end ();
  }
}

void 
Plugin::do_config_end ()
{
  config_finalize ();
  for (tl::Collection<Plugin>::iterator c = m_children.begin (); c != m_children.end (); ++c) {
    c->do_config_end ();
  }
}

bool 
Plugin::do_config_set (const std::string &name, const std::string &value)
{
  if (configure (name, value)) {
    //  taken by us - don't propagate to the children
    return true;
  }

  //  propagate to all children (not only the first that takes it!)
  for (tl::Collection<Plugin>::iterator c = m_children.begin (); c != m_children.end (); ++c) {
    c->do_config_set (name, value);
  }

  return false;
}

// ----------------------------------------------------------------
//  ColorConverter implementation

std::string 
ColorConverter::to_string (const QColor &c)
{
  if (! c.isValid ()) {
    return "auto";
  } else {
    return c.name ().ascii ();
  }
}

void 
ColorConverter::from_string (const std::string &s, QColor &c)
{
  std::string t (tl::trim (s));
  if (t == "auto") {
    c = QColor ();
  } else {
    c = QColor (t.c_str ());
  } 
}

}

==> klayout-0.15.tar.gz: klayout-0.15/src/layPlugin.cc <==

/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include "layPlugin.h"

#include "tlClassRegistry.h"
#include "tlException.h"
#include "tlAssert.h"

//  The instance of the PluginDeclaration class registrar
template <> tl::Registrar<lay::PluginDeclaration> *tl::Registrar<lay::PluginDeclaration>::instance = 0;

namespace lay
{

// ----------------------------------------------------------------
//  Plugin implementation

Plugin::Plugin (Plugin *parent)
  : mp_parent (parent), dm_finalize_config (this, &lay::Plugin::config_end)
{
  if (! parent) {
    //  load the root with the default configuration
    for (tl::Registrar<lay::PluginDeclaration>::iterator cls = tl::Registrar<lay::PluginDeclaration>::begin (); cls != tl::Registrar<lay::PluginDeclaration>::end (); ++cls) {
      std::vector< std::pair<std::string, std::string> > pairs;
      cls->get_options (pairs);
      m_repository.insert (pairs.begin (), pairs.end ());
    }
  } else {
    mp_parent->m_children.insert (this, false /*not owned*/);
  }
}

Plugin::~Plugin ()
{
  // .. nothing yet ..
}

void 
Plugin::config_setup ()
{
  do_config_setup (this);
  do_config_end ();
}
  
void 
Plugin::config_set (const std::string &name, const std::string &value)
{
  //  install the value in the repository
  m_repository.insert (std::make_pair (name, std::string ())).first->second = value;

  //  look for plugins that receive that configuration statically if the root is addressed
  if (! mp_parent) {
    for (tl::Registrar<lay::PluginDeclaration>::iterator cls = tl::Registrar<lay::PluginDeclaration>::begin (); cls != tl::Registrar<lay::PluginDeclaration>::end (); ++cls) {
      if ((const_cast<lay::PluginDeclaration *> (&*cls))->configure (name, value)) {
        return;
      }
    }
  }

  do_config_set (name, value);

  //  schedule a configuration finalization call (once for all config_set calls)
  dm_finalize_config ();
}

void 
Plugin::config_end ()
{
  //  finish static conifgurations for plugins if the root is addressed
  if (! mp_parent) {
    for (tl::Registrar<lay::PluginDeclaration>::iterator cls = tl::Registrar<lay::PluginDeclaration>::begin (); cls != tl::Registrar<lay::PluginDeclaration>::end (); ++cls) {
      (const_cast<lay::PluginDeclaration *> (&*cls))->config_finalize ();
    }
  }

  do_config_end ();
}

void
Plugin::clear_config ()
{
  m_repository.clear ();
  if (! mp_parent) {
    //  load the root with the default configuration
    for (tl::Registrar<lay::PluginDeclaration>::iterator cls = tl::Registrar<lay::PluginDeclaration>::begin (); cls != tl::Registrar<lay::PluginDeclaration>::end (); ++cls) {
      std::vector< std::pair<std::string, std::string> > pairs;
      cls->get_options (pairs);
      m_repository.insert (pairs.begin (), pairs.end ());
    }
  }
  do_config_end ();
}

bool 
Plugin::config_get (const std::string &name, std::string &value) const
{
  std::map<std::string, std::string>::const_iterator p = m_repository.find (name);
  if (p != m_repository.end ()) {
    value = p->second;
    return true;
  } else {
    value = "";
    return false;
  }
}

void 
Plugin::get_config_names (std::vector<std::string> &names)
{
  //  retrieve the configuration names
  for (tl::Registrar<lay::PluginDeclaration>::iterator cls = tl::Registrar<lay::PluginDeclaration>::begin (); cls != tl::Registrar<lay::PluginDeclaration>::end (); ++cls) {

    std::vector< std::pair<std::string, std::string> > pairs;
    cls->get_options (pairs);
    for (std::vector< std::pair<std::string, std::string> >::const_iterator p = pairs.begin (); p != pairs.end (); ++p) {
      names.push_back (p->first);
    }

  }
}

void 
Plugin::do_config_setup (Plugin *target)
{
  if (mp_parent) {
    mp_parent->do_config_setup (target);
  } 
  //  local configurations override the parent's configuration, i.e. are applied after the parents settings
  for (std::map<std::string, std::string>::const_iterator p = m_repository.begin (); p != m_repository.end (); ++p) {
    target->do_config_set (p->first, p->second);
  }
}

void 
Plugin::do_config_end ()
{
  config_finalize ();
  for (tl::Collection<Plugin>::iterator c = m_children.begin (); c != m_children.end (); ++c) {
    c->do_config_end ();
  }
}

bool 
Plugin::do_config_set (const std::string &name, const std::string &value)
{
  if (configure (name, value)) {
    //  taken by us - don't propagate to the children
    return true;
  }

  //  propagate to all children (not only the first that takes it!)
  for (tl::Collection<Plugin>::iterator c = m_children.begin (); c != m_children.end (); ++c) {
    c->do_config_set (name, value);
  }

  return false;
}

// ----------------------------------------------------------------
//  ColorConverter implementation

std::string 
ColorConverter::to_string (const QColor &c)
{
  if (! c.isValid ()) {
    return "auto";
  } else {
    return c.name ().ascii ();
  }
}

void 
ColorConverter::from_string (const std::string &s, QColor &c)
{
  std::string t (tl::trim (s));
  if (t == "auto") {
    c = QColor ();
  } else {
    c = QColor (t.c_str ());
  } 
}

}
